@@ -302,6 +302,7 @@ static int netcfg_activate_static_ipv4(struct debconfclient *client,
 
     buf[sizeof(buf) - 1] = '\0';
 
+    di_info("executing: %s", buf);
     /* NB: unfortunately we cannot use di_exec_shell_log() here, as the active
      * translator would capture its pipe and make it hang forever. */
     rv |= di_exec_shell(buf);
@@ -314,9 +315,11 @@ static int netcfg_activate_static_ipv4(struct debconfclient *client,
 
     /* Flush all previous addresses, routes */
     snprintf(buf, sizeof(buf), "ifconfig %s inet 0 down", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
 
     snprintf(buf, sizeof(buf), "ifconfig %s up", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
 
     snprintf(buf, sizeof(buf), "ifconfig %s %s",
@@ -335,10 +338,18 @@ static int netcfg_activate_static_ipv4(struct debconfclient *client,
     if (!empty_str(interface->pointopoint)) {
         snprintf(buf, sizeof(buf), "route add %s", interface->pointopoint);
         /* avoid using a second buffer */
-        di_snprintfcat(buf, sizeof(buf), "%s", interface->ipaddress);
+        di_snprintfcat(buf, sizeof(buf), " %s", interface->ipaddress);
+        di_info("executing: %s", buf);
         rv |= di_exec_shell_log(buf);
     } else if (!empty_str(interface->gateway)) {
+        if (interface->masklen == 32) {
+            snprintf(buf, sizeof(buf), "route add -interface %s %s", interface->gateway, interface->ipaddress);
+            di_info("executing: %s", buf);
+            rv |= di_exec_shell_log(buf);
+        }
+
         snprintf(buf, sizeof(buf), "route add default %s", interface->gateway);
+        di_info("executing: %s", buf);
         rv |= di_exec_shell_log(buf);
     }
 #else
@@ -349,9 +360,11 @@ static int netcfg_activate_static_ipv4(struct debconfclient *client,
 
     /* Flush all previous addresses, routes */
     snprintf(buf, sizeof(buf), "ip -f inet addr flush dev %s", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
 
     snprintf(buf, sizeof(buf), "ip -f inet route flush dev %s", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
 
     /* Add the new IP address, P-t-P peer (if necessary) and netmask */
@@ -369,10 +382,14 @@ static int netcfg_activate_static_ipv4(struct debconfclient *client,
     if (!empty_str(interface->pointopoint))
     {
         snprintf(buf, sizeof(buf), "ip route add default via %s dev %s", interface->pointopoint, interface->name);
+        di_info("executing: %s", buf);
         rv |= di_exec_shell_log(buf);
     }
     else if (!empty_str(interface->gateway)) {
         snprintf(buf, sizeof(buf), "ip route add default via %s", interface->gateway);
+        if (interface->masklen == 32)
+            di_snprintfcat(buf, sizeof(buf), " dev %s onlink", interface->name);
+        di_info("executing: %s", buf);
         rv |= di_exec_shell_log(buf);
     }
 #endif
@@ -404,6 +421,7 @@ static int netcfg_activate_static_ipv6(struct debconfclient *client,
 
     buf[sizeof(buf) - 1] = '\0';
 
+    di_info("executing: %s", buf);
     /* NB: unfortunately we cannot use di_exec_shell_log() here, as the active
      * translator would capture its pipe and make it hang forever. */
     rv |= di_exec_shell(buf);
@@ -420,6 +438,7 @@ static int netcfg_activate_static_ipv6(struct debconfclient *client,
 
     buf[sizeof(buf) - 1] = '\0';
 
+    di_info("executing: %s", buf);
     rv |= di_exec_shell(buf);
 
 #elif defined(__FreeBSD_kernel__)
@@ -430,9 +449,11 @@ static int netcfg_activate_static_ipv6(struct debconfclient *client,
     
     /* Flush all previous addresses, routes */
     snprintf(buf, sizeof(buf), "ifconfig %s inet 0 down", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
     
     snprintf(buf, sizeof(buf), "ifconfig %s up", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
     
     snprintf(buf, sizeof(buf), "ifconfig %s inet6 %s prefixlen %i",
@@ -442,7 +463,14 @@ static int netcfg_activate_static_ipv6(struct debconfclient *client,
     rv |= di_exec_shell_log(buf);
     
     if (!empty_str(interface->gateway)) {
+        if (interface->masklen == 128) {
+            snprintf(buf, sizeof(buf), "/lib/freebsd/route add -inet6 -interface %s %s", interface->gateway, interface->ipaddress);
+            di_info("executing: %s", buf);
+            rv |= di_exec_shell_log(buf);
+        }
+
         snprintf(buf, sizeof(buf), "/lib/freebsd/route add -inet6 default %s", interface->gateway);
+        di_info("executing: %s", buf);
         rv |= di_exec_shell_log(buf);
     }
 #else
@@ -453,9 +481,11 @@ static int netcfg_activate_static_ipv6(struct debconfclient *client,
 
     /* Flush all previous addresses, routes */
     snprintf(buf, sizeof(buf), "ip -f inet6 addr flush dev %s", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
 
     snprintf(buf, sizeof(buf), "ip -f inet6 route flush dev %s", interface->name);
+    di_info("executing: %s", buf);
     rv |= di_exec_shell_log(buf);
 
     /* Now down and up the interface, to get LL and SLAAC addresses back,
@@ -475,6 +505,8 @@ static int netcfg_activate_static_ipv6(struct debconfclient *client,
 
     if (!empty_str(interface->gateway)) {
         snprintf(buf, sizeof(buf), "ip route add default via %s dev %s", interface->gateway, interface->name);
+        if (interface->masklen == 128)
+            di_snprintfcat(buf, sizeof(buf), " onlink");
         di_info("executing: %s", buf);
         rv |= di_exec_shell_log(buf);
     }
@@ -538,7 +570,7 @@ int netcfg_get_static(struct debconfclient *client, struct netcfg_interface *ifa
 {
     char *nameservers = NULL;
     char *none;
-    char netmask[INET_ADDRSTRLEN];
+    char netmask[NETCFG_ADDRSTRLEN];
 
     enum { BACKUP, GET_HOSTNAME, GET_IPADDRESS, GET_POINTOPOINT, GET_NETMASK,
            GET_GATEWAY, GATEWAY_UNREACHABLE, GET_NAMESERVERS, CONFIRM,
@@ -569,6 +601,8 @@ int netcfg_get_static(struct debconfclient *client, struct netcfg_interface *ifa
                 else if (iface->masklen == 0) {
                     state = GET_NETMASK;
                 } else {
+                    inet_mtop(iface->address_family, iface->masklen, netmask, NETCFG_ADDRSTRLEN);
+                    debconf_set(client, "netcfg/get_netmask", netmask);
                     state = GET_GATEWAY;
                 }
             }
@@ -611,7 +645,7 @@ int netcfg_get_static(struct debconfclient *client, struct netcfg_interface *ifa
             break;
         case GET_NAMESERVERS:
             if (nameservers) free(nameservers);
-            state = (netcfg_get_nameservers (client, &nameservers, iface->gateway)) ?
+            state = netcfg_get_nameservers (client, &nameservers, iface->gateway, iface) ?
                 GET_GATEWAY : CONFIRM;
             break;
         case GET_HOSTNAME:
@@ -641,7 +675,7 @@ int netcfg_get_static(struct debconfclient *client, struct netcfg_interface *ifa
             break;
 
         case CONFIRM:
-            inet_mtop(AF_INET, iface->masklen, netmask, INET_ADDRSTRLEN);
+            inet_mtop(iface->address_family, iface->masklen, netmask, NETCFG_ADDRSTRLEN);
             debconf_subst(client, "netcfg/confirm_static", "interface", iface->name);
             debconf_subst(client, "netcfg/confirm_static", "ipaddress", empty_str(iface->ipaddress) ? none : iface->ipaddress);
             debconf_subst(client, "netcfg/confirm_static", "pointopoint", empty_str(iface->pointopoint) ? none : iface->pointopoint);
